/*
 * Main Loop
 *
 * Copyright (c) 2002  IWAMURO Motonori
 * All rights reserved.
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>

#include "config.h"
#include "init.h"
#include "sigwinch.h"
#include "ccio.h"

#ifndef HAVE_MAX
#define max(a, b) ((a) > (b) ? (a) : (b))
#endif

void
loop(int mfd, FILE *fp, char *term_code, char *proc_code)
{
    CCIO master, slave;
    fd_set fds;
    int fdmax = max(STDIN_FILENO, STDOUT_FILENO) + 1;
    if (fdmax < mfd)
	fdmax = mfd + 1;
    ccio_init(&master, proc_code, term_code);
    ccio_init(&slave,  term_code, proc_code);
    reg_sigwinch(mfd);
    while (1) {
	FD_ZERO(&fds);
	FD_SET(STDIN_FILENO, &fds);
	FD_SET(mfd, &fds);
	if (select(fdmax, &fds, NULL, NULL, NULL) <= 0)
	    continue;
	if (FD_ISSET(STDIN_FILENO, &fds))
	    if (ccio_read(&master, STDIN_FILENO, NULL) != CCIO_SUCCESS)
		break;
	if (ccio_write(&master, mfd) != CCIO_SUCCESS)
	    break;
	if (FD_ISSET(mfd, &fds))
	    if (ccio_read(&slave, mfd, fp) != CCIO_SUCCESS)
		break;
	if (ccio_write(&slave, STDOUT_FILENO) == CCIO_ERROR)
	    break;
    }
    rm_sigwinch();
    ccio_done(&master);
    ccio_done(&slave);
}
